// file      : libcutl/container/multi-index.hxx
// license   : MIT; see accompanying LICENSE file

#ifndef LIBCUTL_CONTAINER_MULTI_INDEX_HXX
#define LIBCUTL_CONTAINER_MULTI_INDEX_HXX

// Multi-index containers support. See the following post for details:
//
// http://www.codesynthesis.com/~boris/blog/2012/09/11/emulating-boost-multi-index-with-std-containers/
//
#include <libcutl/container/key.hxx>
#include <libcutl/container/map-iterator.hxx>

#endif // LIBCUTL_CONTAINER_MULTI_INDEX_HXX
